@@ -1,4 +1,5 @@
 from .pyhackrf_tools import pyhackrf_info, pyhackrf_sweep
+from .libhackrf import pyhackrf
 import argparse
 
 if __name__ == '__main__':
@@ -15,7 +16,7 @@ if __name__ == '__main__':
     pyhackrf_info_parser.add_argument('-s', '--serial_numbers', action='store_true', help='show only founded serial_numbers')
 
     pyhackrf_sweep_parser = subparsers.add_parser(
-        'sweep', help='a command-line spectrum analyzer.', usage='python -m python_hackrf sweep [-h] [-d] [-a] [-f] [-p] [-l] [-g] [-w] [-1] [-N] [-I] [-r]'
+        'sweep', help='a command-line spectrum analyzer.', usage='python -m python_hackrf sweep [-h] [-d] [-a] [-f] [-p] [-l] [-g] [-w] [-1] [-N] [-B] [-s] [-SR] [-r]'
     )
     pyhackrf_sweep_parser.add_argument('-d', action='store', help='serial_number. serial number of desired HackRF', metavar='')
     pyhackrf_sweep_parser.add_argument('-a', action='store_true', help='amp_enable. RX RF amplifier. If specified = Enable')
@@ -23,10 +24,12 @@ if __name__ == '__main__':
     pyhackrf_sweep_parser.add_argument('-p', action='store_true', help='antenna_enable. Antenna port power. If specified = Enable')
     pyhackrf_sweep_parser.add_argument('-l', action='store', help='gain_db. RX LNA (IF) gain, 0-40dB, 8dB steps', metavar='', default=16)
     pyhackrf_sweep_parser.add_argument('-g', action='store', help='gain_db. RX VGA (baseband) gain, 0-62dB, 2dB steps', metavar='', default=20)
-    pyhackrf_sweep_parser.add_argument('-w', action='store', help='bin_width. FFT bin width (frequency resolution) in Hz, 2445-5000000', metavar='', default=1000000)
+    pyhackrf_sweep_parser.add_argument('-w', action='store', help='bin_width. FFT bin width (frequency resolution) in Hz, 245-5000000 Depends on sample rate min= sample rate * 1e6 / 8180, max = sample_rate * 1e6 / 4', metavar='', default=1000000)
     pyhackrf_sweep_parser.add_argument('-1', action='store_true', help='one shot mode. If specified = Enable')
     pyhackrf_sweep_parser.add_argument('-N', action='store', help='num_sweeps. Number of sweeps to perform', metavar='')
     pyhackrf_sweep_parser.add_argument('-B', action='store_true', help='binary output. If specified = Enable')
+    pyhackrf_sweep_parser.add_argument('-s', action='store', help='sweep style ("L" - LINEAR, "I" - INTERLEAVED). Default is INTERLEAVED', metavar='', default='I')
+    pyhackrf_sweep_parser.add_argument('-SR', action='store', help='sample rate (2, 4, 6, 8, 10, 12, 14, 16, 18, 20). Default is 20', metavar='', default=20)
     pyhackrf_sweep_parser.add_argument('-r', action='store', help='filename. output file', metavar='')
     args, unparsed_args = parser.parse_known_args()
 
@@ -64,5 +67,7 @@ if __name__ == '__main__':
                                       binary_output=args.B,
                                       one_shot=args.__dict__.get('1'),
                                       filename=args.r,
+                                      sweep_style=pyhackrf.py_sweep_style.LINEAR if args.s == 'L' else pyhackrf.py_sweep_style.INTERLEAVED,
+                                      sample_rate=int(args.SR) * 1e6,
                                       print_to_console=True,
                                       )
